package com.community.huiju.dao;

import com.community.huiju.model.TpBillOrder;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultMap;
import org.apache.ibatis.annotations.Select;

import java.util.List;

@Mapper
public interface TpBillOrderMapper {
    int deleteByPrimaryKey(Integer id);

    int insert(TpBillOrder record);

    int insertSelective(TpBillOrder record);

    TpBillOrder selectByPrimaryKey(Integer id);

    int updateByPrimaryKeySelective(TpBillOrder record);

    int updateByPrimaryKey(TpBillOrder record);

    /**
     * 批量插入
     * @param list
     * @return
     */
    int batchInsertSelective(List<TpBillOrder> list);

    @ResultMap("BaseResultMap")
    @Select("select * from tp_bill_order where order_bumber=#{orderNumber}")
    List<TpBillOrder> selectByOrderBumber(@Param("orderNumber") String orderNumber);
}